@@ -15,6 +15,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <sys/ioctl.h>
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <unistd.h>
@@ -32,10 +33,14 @@
 
 // Signals we care about are numbered from 1 to 31, inclusive.
 // (32 and above are real-time signals.)
+// TODO: this is likely not portable outside of Linux, or on strange architectures
 #define MAXSIG 31
 
 // Indices are one-indexed (signal 1 is at index 1). Index zero is unused.
+// User-specified signal rewriting.
 int signal_rewrite[MAXSIG + 1] = {[0 ... MAXSIG] = -1};
+// One-time ignores due to TTY quirks. 0 = no skip, 1 = skip the next-received signal.
+char signal_temporary_ignores[MAXSIG + 1] = {[0 ... MAXSIG] = 0};
 
 pid_t child_pid = -1;
 char debug = 0;
@@ -57,7 +62,7 @@ int translate_signal(int signum) {
 
 void forward_signal(int signum) {
     signum = translate_signal(signum);
-    if (signum != -1) {
+    if (signum != 0) {
         kill(use_setsid ? -child_pid : child_pid, signum);
         DEBUG("Forwarded signal %d to children.\n", signum);
     } else {
@@ -87,7 +92,11 @@ void forward_signal(int signum) {
 */
 void handle_signal(int signum) {
     DEBUG("Received signal %d.\n", signum);
-    if (signum == SIGCHLD) {
+
+    if (signal_temporary_ignores[signum] == 1) {
+        DEBUG("Ignoring tty hand-off signal %d.\n", signum);
+        signal_temporary_ignores[signum] = 0;
+    } else if (signum == SIGCHLD) {
         int status, exit_status;
         pid_t killed_pid;
         while ((killed_pid = waitpid(-1, &status, WNOHANG)) > 0) {
@@ -117,7 +126,7 @@ void handle_signal(int signum) {
 
 void print_help(char *argv[]) {
     fprintf(stderr,
-        "dumb-init v%s"
+        "dumb-init v%.*s"
         "Usage: %s [option] command [[arg] ...]\n"
         "\n"
         "dumb-init is a simple process supervisor that forwards signals to children.\n"
@@ -135,7 +144,7 @@ void print_help(char *argv[]) {
         "   -V, --version        Print the current version and exit.\n"
         "\n"
         "Full help is available online at https://github.com/Yelp/dumb-init\n",
-        VERSION,
+        VERSION_len, VERSION,
         argv[0]
     );
 }
@@ -166,8 +175,9 @@ void parse_rewrite_signum(char *arg) {
 }
 
 void set_rewrite_to_sigstop_if_not_defined(int signum) {
-    if (signal_rewrite[signum] == -1)
+    if (signal_rewrite[signum] == -1) {
         signal_rewrite[signum] = SIGSTOP;
+    }
 }
 
 char **parse_command(int argc, char *argv[]) {
@@ -189,7 +199,7 @@ char **parse_command(int argc, char *argv[]) {
                 debug = 1;
                 break;
             case 'V':
-                fprintf(stderr, "dumb-init v%s", VERSION);
+                fprintf(stderr, "dumb-init v%.*s", VERSION_len, VERSION);
                 exit(0);
             case 'c':
                 use_setsid = 0;
@@ -233,12 +243,55 @@ char **parse_command(int argc, char *argv[]) {
     return &argv[optind];
 }
 
+// A dummy signal handler used for signals we care about.
+// On the FreeBSD kernel, ignored signals cannot be waited on by `sigwait` (but
+// they can be on Linux). We must provide a dummy handler.
+// https://lists.freebsd.org/pipermail/freebsd-ports/2009-October/057340.html
+void dummy(int signum) {}
+
 int main(int argc, char *argv[]) {
     char **cmd = parse_command(argc, argv);
     sigset_t all_signals;
     sigfillset(&all_signals);
     sigprocmask(SIG_BLOCK, &all_signals, NULL);
 
+    int i = 0;
+    for (i = 1; i <= MAXSIG; i++) {
+        signal(i, dummy);
+    }
+
+    /*
+     * Detach dumb-init from controlling tty, so that the child's session can
+     * attach to it instead.
+     *
+     * We want the child to be able to be the session leader of the TTY so that
+     * it can do normal job control.
+     */
+    if (use_setsid) {
+        if (ioctl(STDIN_FILENO, TIOCNOTTY) == -1) {
+            DEBUG(
+                "Unable to detach from controlling tty (errno=%d %s).\n",
+                errno,
+                strerror(errno)
+            );
+        } else {
+            /*
+             * When the session leader detaches from its controlling tty via
+             * TIOCNOTTY, the kernel sends SIGHUP and SIGCONT to the process
+             * group. We need to be careful not to forward these on to the
+             * dumb-init child so that it doesn't receive a SIGHUP and
+             * terminate itself (#136).
+             */
+            if (getsid(0) == getpid()) {
+                DEBUG("Detached from controlling tty, ignoring the first SIGHUP and SIGCONT we receive.\n");
+                signal_temporary_ignores[SIGHUP] = 1;
+                signal_temporary_ignores[SIGCONT] = 1;
+            } else {
+                DEBUG("Detached from controlling tty, but was not session leader.\n");
+            }
+        }
+    }
+
     child_pid = fork();
     if (child_pid < 0) {
         PRINTERR("Unable to fork. Exiting.\n");
@@ -255,6 +308,14 @@ int main(int argc, char *argv[]) {
                 );
                 exit(1);
             }
+
+            if (ioctl(STDIN_FILENO, TIOCSCTTY, 0) == -1) {
+                DEBUG(
+                    "Unable to attach to controlling tty (errno=%d %s).\n",
+                    errno,
+                    strerror(errno)
+                );
+            }
             DEBUG("setsid complete.\n");
         }
         execvp(cmd[0], &cmd[0]);
@@ -265,6 +326,11 @@ int main(int argc, char *argv[]) {
     } else {
         /* parent */
         DEBUG("Child spawned with PID %d.\n", child_pid);
+        if (chdir("/") == -1) {
+             DEBUG("Unable to chdir(\"/\") (errno=%d %s)\n",
+                   errno,
+                   strerror(errno));
+        }
         for (;;) {
             int signum;
             sigwait(&all_signals, &signum);
