@@ -8,6 +8,7 @@ from subprocess import Popen
 import pytest
 
 from testing import is_alive
+from testing import kill_if_alive
 from testing import pid_tree
 from testing import sleep_until
 
@@ -16,7 +17,7 @@ def spawn_and_kill_pipeline():
     proc = Popen((
         'dumb-init',
         'sh', '-c',
-        "yes 'oh, hi' | tail & yes error | tail >&2"
+        "yes 'oh, hi' | tail & yes error | tail >&2",
     ))
 
     def assert_living_pids():
@@ -31,7 +32,7 @@ def spawn_and_kill_pipeline():
 
 
 def living_pids(pids):
-    return set(pid for pid in pids if is_alive(pid))
+    return {pid for pid in pids if is_alive(pid)}
 
 
 @pytest.mark.usefixtures('both_debug_modes', 'setsid_enabled')
@@ -60,7 +61,7 @@ def test_no_setsid_doesnt_signal_entire_group():
     sleep_until(assert_four_living_pids)
 
     for pid in living_pids(pids):
-        os.kill(pid, signal.SIGKILL)
+        kill_if_alive(pid)
 
 
 def spawn_process_which_dies_with_children():
@@ -79,7 +80,7 @@ def spawn_process_which_dies_with_children():
             # we need to sleep before the shell exits, or dumb-init might send
             # TERM to print_signals before it has had time to register custom
             # signal handlers
-            '{python} -m testing.print_signals & sleep 0.1'.format(
+            '{python} -m testing.print_signals & sleep 1'.format(
                 python=sys.executable,
             ),
         ),
@@ -90,7 +91,7 @@ def spawn_process_which_dies_with_children():
 
     # read a line from print_signals, figure out its pid
     line = proc.stdout.readline()
-    match = re.match(b'ready \(pid: ([0-9]+)\)\n', line)
+    match = re.match(b'ready \\(pid: ([0-9]+)\\)\n', line)
     assert match, line
     child_pid = int(match.group(1))
 
@@ -128,12 +129,14 @@ def test_processes_dont_receive_term_on_exit_if_no_setsid():
     os.kill(child_pid, signal.SIGKILL)
 
 
-@pytest.mark.parametrize('args', [
-    ('/doesnotexist',),
-    ('--', '/doesnotexist'),
-    ('-c', '/doesnotexist'),
-    ('--single-child', '--', '/doesnotexist'),
-])
+@pytest.mark.parametrize(
+    'args', [
+        ('/doesnotexist',),
+        ('--', '/doesnotexist'),
+        ('-c', '/doesnotexist'),
+        ('--single-child', '--', '/doesnotexist'),
+    ],
+)
 @pytest.mark.usefixtures('both_debug_modes', 'both_setsid_modes')
 def test_fails_nonzero_with_bad_exec(args):
     """If dumb-init can't exec as requested, it should exit nonzero."""
